"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RestClientError = void 0;
const axios_1 = __importDefault(require("axios"));
const http_1 = require("http");
const https_1 = require("https");
const constants_1 = require("../constants");
class RestClientError extends Error {
    constructor(res, message) {
        super(message);
        this.response = res;
    }
}
exports.RestClientError = RestClientError;
class RestClient {
    constructor(workspaceConfig, workspaceName) {
        this.clientHeaders = workspaceConfig.headers || {};
        this.workspaceName = workspaceName;
        if (workspaceConfig.kongAdminUrl) {
            this.clientUrl = `${workspaceConfig.kongAdminUrl}`;
        }
        else if (workspaceConfig.upstream) {
            console.log('upstream is deprecated and will cease to function in a later release. Please use kong_admin_url (upstream url without the workspace suffix)');
            const match = workspaceConfig.upstream.match(/.*\//);
            if (!match) {
                console.log('unable to parse upstream url');
                throw new Error();
            }
            this.clientUrl = match[0];
        }
        else {
            console.log('kongAdminUrl not set');
            throw new Error();
        }
        if (workspaceConfig.kongAdminToken) {
            this.clientHeaders['Kong-Admin-Token'] = workspaceConfig.kongAdminToken;
        }
        let httpsAgent;
        if (workspaceConfig.disableSSLVerification) {
            httpsAgent = new https_1.Agent({
                rejectUnauthorized: false,
                keepAlive: true,
            });
        }
        else {
            httpsAgent = new https_1.Agent({
                keepAlive: true,
            });
        }
        this.client = axios_1.default.create({
            baseURL: this.clientUrl,
            headers: this.clientHeaders,
            httpAgent: new http_1.Agent({ keepAlive: true }),
            maxContentLength: (workspaceConfig.maxContentLengthInMb ? workspaceConfig.maxContentLengthInMb : constants_1.MAX_CONTENT_LENGTH_MB) * constants_1.ONE_MB,
            httpsAgent,
        });
    }
    async getFiles(options) {
        return this.handleResponse(await this.client.get(`${this.workspaceName}/files`, options));
    }
    async getAllFiles(params = {}) {
        let res = await this.client.get(`${this.workspaceName}/files`, {
            params,
        });
        let files = this.handleResponse(res);
        while (res.data.next) {
            res = await this.client.get(res.data.next);
            files = files.concat(this.handleResponse(res));
        }
        return files;
    }
    async saveFile(file, options = {}) {
        try {
            await this.client.put(`${this.workspaceName}/files/${file.path}`, file, options);
        }
        catch (e) {
            console.log(`\n\nError uploading file: ${file.path}\n`);
            console.error((e.response && e.response.data) || e);
            throw e;
        }
    }
    async deleteFile(file, options = {}) {
        try {
            await this.client.delete(`${this.workspaceName}/files/${file.path}`, options);
        }
        catch (e) {
            console.log(`\n\nError deleting file: ${file.path}\n`);
            console.error((e.response && e.response.data) || e);
            throw e;
        }
    }
    async enablePortal(options = {}) {
        await this.client.patch(`${this.workspaceName}/workspaces/${this.workspaceName}`, { config: { portal: true } }, options);
    }
    async disablePortal(options = {}) {
        await this.client.patch(`${this.workspaceName}/workspaces/${this.workspaceName}`, { config: { portal: false } }, options);
    }
    handleResponse(res) {
        if (!res.data) {
            console.log('malformed server reply');
            throw new Error();
        }
        return res.data.data;
    }
    handleError(err) {
        if (err.message === 'Request body larger than maxBodyLength limit') {
            return '\nFiles must be less than 10mb';
        }
        if (err.response && err.response.status === 401) {
            return err.message + '\n\t Make sure you have the correct admin token and RBAC settings for that token';
        }
        if (err.response && err.response.status === 404) {
            return err.message + '\n\t Make sure Portal is enabled on this workspace: \n\t "portal enable <workspace>"';
        }
        return err.message;
    }
}
exports.default = RestClient;
